use derive_setters::Setters;
use ratatui::{
    layout::{
        Constraint,
        Direction,
        Layout,
        Rect
    },
    style::{Color, Style, Stylize},
    text::{Line, Span, Text},
    widgets::{Block, Borders, Clear, List, ListItem, Paragraph, Widget, Wrap},
    Frame,
};
use ratatui::crossterm::{event::{self, Event, KeyCode, KeyEvent, KeyEventKind}};

use crate::{client::Client, player::Player};

#[derive(Debug, Default, Setters)]
struct Popup<'a> {
    #[setters(into)]
    title: Line<'a>,
    #[setters(into)]
    content: Text<'a>,
    border_style: Style,
    title_style: Style,
    style: Style,
}

impl Widget for Popup<'_> {
    fn render(self, area: Rect, buf: &mut ratatui::prelude::Buffer)
        where
            Self: Sized {
        Clear.render(area, buf);

        let block = Block::new()
            .title(self.title)
            .title_style(self.title_style)
            .borders(Borders::ALL)
            .border_style(self.border_style);

        Paragraph::new(self.content)
            .wrap(Wrap { trim: true })
            .style(self.style)
            .block(block)
            .render(area, buf);
    }
}

async fn handle_key_event(mut client: &mut Client, key_event: KeyEvent) {
    match &client.popup {
        false => {
            match key_event.code {
                KeyCode::Char('q') => Client::exit(client),
                KeyCode::Char('1') => {
                    if !client.name.is_empty() {
                        client.popup_title = "Error".to_string();
                        client.popup_content = "user already_created".to_string();
                        client.popup = true;
                        return
                    }

                    client.user = Some(Player::new().expect("toto"));
                    client.name = client.user.clone().unwrap().name;
                },
                KeyCode::Char('2') => {
                    todo!()
                },
                KeyCode::Char('3') => {
                    if let Some(player) = client.user.clone() {
                        let addr = client.addr.clone();
                        Client::send_player(&mut client, &player, &addr).await.unwrap();
                        return
                    }
                }
                _ => (),
            }
        }
        true => {
            match key_event.code {
                _ => client.popup = false,
            }
        }
    }
}

pub async fn handle_events(client: &mut Client) -> anyhow::Result<()> {
    match event::read()? {
        Event::Key(key) if key.kind == KeyEventKind::Press => {
            handle_key_event(client, key).await
        }
        _ => {}
    };

    Ok(())
}

pub fn ui(frame: &mut Frame, app: &Client) {
    let chunks = Layout::default()
        .direction(Direction::Vertical)
        .constraints([
            Constraint::Min(3),
            Constraint::Min(1),
        ])
        .split(frame.area());

    let instructions = Line::from(vec![
        " 1".blue().bold(),
        " create player ".into(),
        "2".blue().bold(),
        " create lobby ".into(),
        "3".blue().bold(),
        " send player ".into(),
        "<Q>".blue().bold(),
        " quit ".into(),
    ]);

    let infos = Text::from(vec![
        Line::from(vec![
            "connected to: ".into(),
            app.addr.clone().yellow(),
        ]),
        Line::from(vec![
            "username: ".into(),
            app.name.clone().yellow(),
        ]),
    ]);

    let title = Line::from(" durak tui test fre ".bold());
    let top_block = Block::default()
        .title(title.centered())
        .borders(Borders::ALL)
        .style(Style::default());
    frame.render_widget(Paragraph::new(infos).block(top_block), chunks[0]);

    let server_message = Text::from(vec![
        Line::from(vec![
            "message: ".into(),
            app.response.clone().blue(),
        ])
    ]);

    let btm_block = Block::default()
        .title_bottom(instructions.centered())
        .borders(Borders::ALL)
        .style(Style::default());
    frame.render_widget(
        Paragraph::new(server_message)
            .wrap(Wrap { trim: true, })
            .block(btm_block),
        chunks[1],
    );

    let popup_area = Rect {
        x: frame.area().width / 4,
        y: frame.area().height / 3,
        width: frame.area().width / 2,
        height: frame.area().height / 3,
    };

    match &app.popup {
        true => {
            let popup = Popup::default()
                .title(app.popup_title.clone())
                .content(app.popup_content.clone())
                .style(Style::new().yellow())
                .title_style(Style::new().white().bold())
                .border_style(Style::new().red());
            frame.render_widget(popup, popup_area);
        },
        _ => ()
    }
}
